
#include <Adafruit_GFX.h>      // Include core graphics library for the display
#include <Adafruit_SSD1306.h>  // Include Adafruit_SSD1306 library to drive the display


Adafruit_SSD1306 display(128, 64);  // Create display

#include <Fonts/FreeMonoBold12pt7b.h>  
#include <Fonts/FreeMono9pt7b.h>       

// Back sensor variables
int B1x = 27, B2x = 15, B3x = 3;
int B1y0 = 17, B1y1 = 47;
int B2y0 = B1y0 - 3;
int B2y1 = B1y1 + 3;
int B3y0 = B2y0 - 3;
int B3y1 = B2y1 + 3;

// Front sensor variables
int F1x = 101, F2x = 113, F3x = 125;
int F1y0 = 17, F1y1 = 47;
int F2y0 = F1y0 - 3;
int F2y1 = F1y1 + 3;
int F3y0 = F2y0 - 3;
int F3y1 = F2y1 + 3;

const int P_BUZZER = 9;

const int P_LED_GREEN = 10;
const int P_LED_YELLOW = 11;
const int P_LED_ORANGE = 12;
const int P_LED_RED = 13;

const int P_FS_TRIG = 3;
const int P_FS_ECHO = 2;
const int P_BS_TRIG = 5;
const int P_BS_ECHO = 4;

int RED_state = 0;
int ORANGE_state = 0;
int YELLOW_state = 0;
int GREEN_state = 0;

int BUZZER_frequency = 2;
int BUZZER_state = 0;

float front_duration, front_distance;
float back_duration, back_distance;

int calculateTopValue(int frequency) {
  return 16000000 / (256 * frequency) - 1;
}

ISR(TIMER1_COMPA_vect) {
  BUZZER_state = (BUZZER_state == 0) ? 1 : 0;
  digitalWrite(P_BUZZER, BUZZER_state);

  OCR1A = calculateTopValue(BUZZER_frequency);
}

void configure_timer1() {
  // configure timer1 in CTC mode with 256 prescaler
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  OCR1A = calculateTopValue(BUZZER_frequency);
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS12);
}

void init_timer1() {
  // enable timer compare interrupt
  TIMSK1 |= (1 << OCIE1A);
}

void setup() {
  delay(100);  // This delay is needed to let the display to initialize

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // Initialize display with the I2C address of 0x3C

  display.clearDisplay();  // Clear the buffer

  display.setTextColor(WHITE);  // Set color of the text

  display.setRotation(0);  // Set orientation. Goes from 0, 1, 2 or 3

  display.setTextWrap(false);  // By default, long lines of text are set to automatically “wrap” back to the leftmost column.
                               // To override this behavior (so text will run off the right side of the display - useful for
                               // scrolling marquee effects), use setTextWrap(false). The normal wrapping behavior is restored
                               // with setTextWrap(true).

  display.dim(0);

  pinMode(P_LED_GREEN, OUTPUT);
  pinMode(P_LED_YELLOW, OUTPUT);
  pinMode(P_LED_ORANGE, OUTPUT);
  pinMode(P_LED_RED, OUTPUT);

  pinMode(P_BUZZER, OUTPUT);

  pinMode(P_FS_TRIG, OUTPUT);
  pinMode(P_FS_ECHO, INPUT);

  pinMode(P_BS_TRIG, OUTPUT);
  pinMode(P_BS_ECHO, INPUT);

  cli();
  configure_timer1();
  init_timer1();
  // activate global interrupts
  sei();

  Serial.begin(9600);
}


void loop() {
  display.clearDisplay();

  if (RED_state == 0) {
    display.drawRoundRect(39, 17, 50, 30, 8, WHITE);
    display.drawTriangle(84, 32, 70, 22, 70, 42, WHITE);
  }

  // leds
  digitalWrite(P_LED_GREEN, GREEN_state);
  digitalWrite(P_LED_YELLOW, YELLOW_state);
  digitalWrite(P_LED_ORANGE, ORANGE_state);
  digitalWrite(P_LED_RED, RED_state);

  // front sensor
  digitalWrite(P_FS_TRIG, LOW);
  delayMicroseconds(2);
  digitalWrite(P_FS_TRIG, HIGH);
  delayMicroseconds(10);
  digitalWrite(P_FS_TRIG, LOW);

  front_duration = pulseIn(P_FS_ECHO, HIGH);

  front_distance = (front_duration * .0343) / 2;
  Serial.print("Front Distance: ");
  Serial.println(front_distance);

  // back sensor
  digitalWrite(P_BS_TRIG, LOW);
  delayMicroseconds(2);
  digitalWrite(P_BS_TRIG, HIGH);
  delayMicroseconds(10);
  digitalWrite(P_BS_TRIG, LOW);

  back_duration = pulseIn(P_BS_ECHO, HIGH);

  back_distance = (back_duration * .0343) / 2;
  Serial.print("Back Distance: ");
  Serial.println(back_distance);
  // closer = the closer distance detected from the 2 sensors
  float closer = (front_distance > back_distance) ? back_distance : front_distance;
  // front_or_back = check where is the closer distance detected
  // front = 1
  // back = 0
  int front_or_back = (front_distance > back_distance) ? 0 : 1;
  // set the led and buzzer modes
  if (closer > 20) {
    GREEN_state = 1;
    YELLOW_state = 0;
    ORANGE_state = 0;
    RED_state = 0;
    BUZZER_frequency = 2;
  } else if (closer > 14) {
    GREEN_state = 0;
    YELLOW_state = 1;
    ORANGE_state = 0;
    RED_state = 0;
    BUZZER_frequency = 4;
  } else if (closer > 8) {
    GREEN_state = 0;
    YELLOW_state = 0;
    ORANGE_state = 1;
    RED_state = 0;
    BUZZER_frequency = 8;
  } else {
    GREEN_state = 0;
    YELLOW_state = 0;
    ORANGE_state = 0;
    RED_state = 1;
    // draw red mode scheme
    display.drawRoundRect(14, 12, 30, 40, 8, WHITE);
    if (front_or_back == 1) {
      display.drawTriangle(40, 32, 25, 20, 25, 44, WHITE);
    } else {
      display.drawTriangle(18, 32, 33, 20, 33, 44, WHITE);
    }

    if (closer > 6) {
      display.drawLine(89, 4, 89, 57, WHITE);
      display.drawLine(89, 4, 89 - 5, 4 - 3, WHITE);
      display.drawLine(89, 57, 89 - 5, 57 + 3, WHITE);
    } else if (closer > 4) {
      display.drawLine(74, 7, 74, 54, WHITE);
      display.drawLine(74, 7, 74 - 5, 7 - 3, WHITE);
      display.drawLine(74, 54, 74 - 5, 54 + 3, WHITE);
    } else {
      display.drawLine(59, 10, 59, 51, WHITE);
      display.drawLine(59, 10, 54, 7, WHITE);
      display.drawLine(59, 51, 54, 54, WHITE);
    }
    BUZZER_frequency = 16;
  }
  // draw front and back lines
  if (RED_state == 0) {
    if (back_distance > 20) {
      display.drawLine(B3x, B3y0, B3x, B3y1, WHITE);
      display.drawLine(B3x, B3y0, B3x + 10, B3y0 - 7, WHITE);
      display.drawLine(B3x, B3y1, B3x + 10, B3y1 + 7, WHITE);
    } else if (back_distance > 14) {
      display.drawLine(B2x, B2y0, B2x, B2y1, WHITE);
      display.drawLine(B2x, B2y0, B2x + 10, B2y0 - 7, WHITE);
      display.drawLine(B2x, B2y1, B2x + 10, B2y1 + 7, WHITE);
    } else if (back_distance > 8) {
      display.drawLine(B1x, B1y0, B1x, B1y1, WHITE);
      display.drawLine(B1x, B1y0, B1x + 10, B1y0 - 7, WHITE);
      display.drawLine(B1x, B1y1, B1x + 10, B1y1 + 7, WHITE);
    }
    if (front_distance > 20) {
      display.drawLine(F3x, F3y0, F3x, F3y1, WHITE);
      display.drawLine(F3x, F3y0, F3x - 10, F3y0 - 7, WHITE);
      display.drawLine(F3x, F3y1, F3x - 10, F3y1 + 7, WHITE);
    } else if (front_distance > 14) {
      display.drawLine(F2x, F2y0, F2x, F2y1, WHITE);
      display.drawLine(F2x, F2y0, F2x - 10, F2y0 - 7, WHITE);
      display.drawLine(F2x, F2y1, F2x - 10, F2y1 + 7, WHITE);
    } else if (front_distance > 8) {
      display.drawLine(F1x, F1y0, F1x, F1y1, WHITE);
      display.drawLine(F1x, F1y0, F1x - 10, F1y0 - 7, WHITE);
      display.drawLine(F1x, F1y1, F1x - 10, F1y1 + 7, WHITE);
    }
  }

  display.display();

  delay(100);
}